function [cc,Iter_AE,p_hat,v_hat,rank] = DSFLocUknPropSpdMvgSrc_AE(s,s_dot,f,Qe,f0)
% function [cc,Iter_AE,p_hat,v_hat,rank] = DSFLocUknPropSpdMvgSrc_AE(s,s_dot,f,Qe,f0)
%
% AE method for the moving source localization by frequency measurements
% when signal propagation speed is not known.
% 
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   Qe:       (N x N), the covariance matrix of measurement noise
%   f0:       (1 x 1), the carrier frequency
%
% Output:
%   
%   cc:       (1 x 1), the estimate of signal propagation speed
%   Iter_AE:  (1 x 1), the iteration times of AE method
%   p_hat:    (2 x 1), the estimate of source position
%   v_hat:    (2 x 1), the estimate of source velocity
%   rank:     (1 x 1), the rank of solution of AE-subproblem1 in the last iteration
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency 
% measurements in unknown signal propagation speed environments," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	Nov. 1, 2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f); %number of sensors
c0 = 1.5;  %initial signal propagation speed (km/s)
% ============================step 1=======================================
Iter_AE = 0; %to count the number of iteration times
W = inv(Qe); %fix the weighting matrix to Q^-1 in step 1
cc = 0;

while abs(c0-cc)>1e-2&&Iter_AE<50
    cc = c0;
    [p_hat,v_hat,rank] = DSFLocUknPropSpdMvgSrc_AESubPlbm1(s,s_dot,f,f0,c0,W); %solution of AE-subproblem1
    for i=1:N
        r(i) = -(f(i)-f0)/f0;
        a(i) = r(i)^2*norm(s(:,i))^2+r(i)^2*norm(p_hat)^2-2*r(i)^2*s(:,i)'*p_hat;
        g(i) = -(2*s(:,i)'*s_dot(:,i)*s_dot(:,i)'*p_hat+2*s(:,i)'*s_dot(:,i)*s(:,i)'*v_hat ...
            -2*s(:,i)'*s_dot(:,i)*p_hat'*v_hat-2*s(:,i)'*v_hat*p_hat'*s_dot(:,i)-(s_dot(:,i)'*p_hat)^2 ...
            -(s(:,i)'*v_hat)^2+2*s_dot(:,i)'*p_hat*p_hat'*v_hat+2*s(:,i)'*v_hat*p_hat'*v_hat ...
            -(p_hat'*v_hat)^2-(s(:,i)'*s_dot(:,i))^2);
    end

    c0 = sqrt(inv(a*W*a')*a*W*g');
    Iter_AE=Iter_AE+1;
end

% ============================step 2=======================================
cc = 1.5;%initial signal propagation speed (km/s)
Iter = 0;

while abs(c0-cc)>1e-5&&Iter<50
    cc = c0;
    B = [];
    for j=1:N
        B = blkdiag(B,2*r(j)*c0^2/f0*norm(p_hat-s(:,j))^2);
    end
    W = inv(B'*Qe*B); %update the weighting matrix
    [p_hat,v_hat,rank] = DSFLocUknPropSpdMvgSrc_AESubPlbm1(s,s_dot,f,f0,c0,W); %solution of AE-subproblem1
    for i=1:N
        r(i) = -(f(i)-f0)/f0;
        a(i) = r(i)^2*norm(s(:,i))^2+r(i)^2*norm(p_hat)^2-2*r(i)^2*s(:,i)'*p_hat;
        g(i) = -(2*s(:,i)'*s_dot(:,i)*s_dot(:,i)'*p_hat+2*s(:,i)'*s_dot(:,i)*s(:,i)'*v_hat ...
            -2*s(:,i)'*s_dot(:,i)*p_hat'*v_hat-2*s(:,i)'*v_hat*p_hat'*s_dot(:,i)-(s_dot(:,i)'*p_hat)^2 ...
            -(s(:,i)'*v_hat)^2+2*s_dot(:,i)'*p_hat*p_hat'*v_hat+2*s(:,i)'*v_hat*p_hat'*v_hat ...
            -(p_hat'*v_hat)^2-(s(:,i)'*s_dot(:,i))^2);
    end
    c0 = sqrt(inv(a*W*a')*a*W*g');

    Iter = Iter+1;
end

Iter_AE = Iter_AE + Iter;

cc = c0;

end


% #########################################################################################
% #########################################################################################
function [p,v,rank] = DSFLocUknPropSpdMvgSrc_AESubPlbm1(s,s_dot,f,f0,c,W)
% function [p,v,rank] = DSFLocUknPropSpdMvgSrc_AESubPlbm1(s,s_dot,f,f0,c,W)
%
% AE-subproblem1 for the moving source localization by frequency
% measurements when signal propagation speed is not known.
%
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   f0:       (1 x 1), the carrier frequency
%   c:        (1 x 1), signal propagation speed
%   W:        (N x N), weighting matrx
%
% Output:
%   p:       (2 x 1), the estimate of source position
%   v:       (2 x 1), the estimate of source velocity
%   rank:    (1 x 1), the rank of SDP solution
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency 
% measurements in unknown signal propagation speed environments," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f);
scaling = 10; %scaling factor
s = s*scaling;
s_dot = s_dot*scaling;
c = c*scaling;

b = [];
d = [];

for i=1:N
    d(i) = -c*(f(i)-f0)/f0;
    A(i,:) = -[2*s(:,i)'*s_dot(:,i)*s_dot(:,i)'-2*d(i)^2*s(:,i)',2*s(:,i)'*s_dot(:,i)*s(:,i)',d(i)^2, ...
        -(2*s(:,i)'*s_dot(:,i)+2*s_dot(1,i)*s(1,i)),-(2*s(:,i)'*s_dot(:,i)+2*s_dot(2,i)*s(2,i)), ...
        -2*s_dot(1,i)*s(2,i),-2*s_dot(2,i)*s(1,i),-s_dot(1,i)^2,-2*s_dot(1,i)*s_dot(2,i), ...
        -s_dot(2,i)^2,-s(1,i)^2,-2*s(1,i)*s(2,i),-s(2,i)^2,2*s_dot(:,i)',2*s(:,i)',-1];
    b = [b;d(i)^2*norm(s(:,i))^2-(s(:,i)'*s_dot(:,i))^2];
end

W = W/trace(W);
cvx_clear
cvx_begin sdp
cvx_solver sdpt3
cvx_quiet(1)
cvx_precision best
variable Y(20,20) symmetric
variable y(20,1)

minimize ((trace(A'*W*A*Y)-2*b'*W*A*y+b'*W*b)*1e5)%

subject to
[Y y;y' 1]>=0;
y(5) == Y(1,1)+Y(2,2);
y(6) == Y(1,3);
y(7) == Y(2,4);
y(8) == Y(1,4);
y(9) == Y(2,3);
y(10) == Y(1,1);
y(11) == Y(1,2);
y(12) == Y(2,2);
y(13) == Y(3,3);
y(14) == Y(3,4);
y(15) == Y(4,4);
y(16) == Y(3,10)+Y(4,11);
y(17) == Y(3,11)+Y(4,12);
y(18) == Y(1,13)+Y(2,14);
y(19) == Y(1,14)+Y(2,15);
y(20) == Y(3,16)+Y(4,17);
Y(1,6) == Y(3,10);
Y(1,7) == Y(4,11);
Y(1,9) == Y(3,11);
Y(1,13) == Y(3,6);
Y(1,14) == Y(4,6);
Y(1,14) == Y(3,8);
Y(1,18) == Y(3,16);
Y(2,19) == Y(4,17);
Y(2,6) == Y(3,11);
Y(2,7) == Y(4,12);
Y(2,8) == Y(4,11);
Y(2,14) == Y(3,7);
Y(2,14) == Y(4,9);
Y(2,15) == Y(4,7);
Y(1,16) == Y(6,10)+Y(8,11);
Y(7,10) == Y(8,11);
Y(2,16) == Y(9,10)+Y(8,12);
Y(6,11) == Y(9,10);
Y(7,11) == Y(8,12);
Y(4,16) == Y(6,8)+Y(11,15);
Y(7,8) == Y(11,15);
Y(2,17) == Y(9,11)+Y(7,12);
Y(6,12) == Y(9,11);
Y(1,18) == Y(10,13)+Y(6,7);
Y(10,13) == Y(6,6);
Y(6,7) == Y(11,14);
Y(6,7) == Y(8,9);
Y(2,18) == Y(11,13)+Y(12,14);
Y(6,9) == Y(11,13);
Y(4,18) == Y(8,13)+Y(7,14);
Y(6,14) == Y(8,13);
Y(7,14) == Y(9,15);
Y(2,19) == Y(8,9)+Y(7,7);
Y(1,20) == Y(6,16)+Y(8,17);
Y(6,16) == Y(10,18);
Y(7,16) == Y(8,17);
Y(7,16) == Y(11,19);
Y(2,20) == Y(9,16)+Y(7,17);
Y(6,17) == Y(9,16);
Y(6,17) == Y(11,18);
Y(7,17) == Y(12,19);
Y(3,20) == Y(6,18)+Y(9,19);
Y(6,18) == Y(13,16);
Y(7,18) == Y(9,19);
Y(7,18) == Y(14,17);
Y(4,20) == Y(14,16)+Y(15,17);
Y(6,19) == Y(14,16);
Y(6,19) == Y(8,18);
Y(7,19) == Y(15,17);
Y(3,5) == Y(1,6)+Y(3,12);
Y(2,9) == Y(3,12);
Y(4,5) == Y(4,10)+Y(2,7);
Y(1,8) == Y(4,10);
Y(5,14) == Y(10,14)+Y(7,9);
Y(6,8) == Y(10,14);
Y(7,9) == Y(12,14);
Y(5,15) == Y(8,8)+Y(12,15);
Y(12,15) == Y(7,7);
Y(10,15) == Y(8,8);

cvx_end


rank=0;
lambda=[];
lambda=eig(Y);
if lambda(20)/lambda(19)>=1e5
    rank=1;
end

p = y(1:2)/scaling;
v = y(3:4)/scaling;

end

