function CRLB = DSFLocUknPropSpdMvgSrc_CRLB(p,v,s,s_dot,f0,c,Qe)
% function CRLB = DSFLocUknPropSpdMvgSrc_CRLB(p,v,s,s_dot,f0,c,Qe)
%
% CRLB for moving source localization in 2-D by frequency measurements when 
% signal propagation speed is not known.
%
% Input:
%   p:      (2 x 1), the true value of source position
%   v:      (2 x 1), the true value of source velocity
%   s:      (2 x N), the true value of sensor positions, N = number of sensors 
%   s_dot:  (2 x N), the true value of sensor velocities
%   f0:     (1 x 1), the true value of carrier frequency
%   c:      (1 x 1), the true value of signal propagation speed
%   Qe:     (N x N), the covariance matrix of the masurement noise
%
% Output:
%   CRLB:  (5 x 1), CRLB matrix of all unknowns for the moving source localization 
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency 
% measurements in unknown signal propagation speed environments," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	Nov. 1, 2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = size(s,2);   %number of sensors
I = eye(2,2);
% initialization
a = zeros(N,2);  %partial derivative of the frequency measurements to p
b = zeros(N,2);  %partial derivative of the frequency measurements to v
de_c = zeros(N,1);  %partial derivative of the frequency measurements to c
de_fo = zeros(N,1);


for i=1:N
    a(i,:) = -f0/c*(v-s_dot(:,i))'/(norm(p-s(:,i)))*(I-(p-s(:,i))*(p-s(:,i))'/(norm(p-s(:,i))^2));
    b(i,:) = -f0/c*(p-s(:,i))'/norm(p-s(:,i));
    de_c(i,:) = f0/c^2*(p-s(:,i))'*(v-s_dot(:,i))/norm(p-s(:,i));
end

f_theta = [a b de_c];
X=f_theta'/Qe*f_theta; % FIM

CRLB= diag(inv(X));


