function [p1,c1,v1,rank] = DSFLocUknPropSpdMvgSrc_SDR(s,s_dot,f,Q,f0)
% function [p1,c1,v1,rank] = DSFLocUknPropSpdMvgSrc_SDR(s,s_dot,f,Q,f0)
%
% This rountine performs moving source localization using SDP by SDR, 
% when the signal propagation speed is not known.  
%
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   Q:        (N x N), the covariance matrix of measurement noise
%   f0:       (1 x 1), the carrier frequency
%
% Output:
%   p1:       (2 x 1), the estimate of source position
%   c1:       (1 x 1), the estimate of signal propagation speed
%   v1:       (2 x 1), the estimate of source velocity
%   rank:     (1 x 1), the rank of SDP solution
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency 
% measurements in unknown signal propagation speed environments," IEEE 
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	Nov. 1, 2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f);

b = [];
d = [];

for i=1:N
    d(i) = -(f(i)-f0)/f0;
    A(i,:) = [2*s(:,i)'*s_dot(:,i)*s_dot(:,i)',2*s(:,i)'*s_dot(:,i)*s(:,i)',d(i)^2*norm(s(:,i))^2,d(i)^2,-2*d(i)^2*s(:,i)', ...
        -2*(s_dot(1,i)*s(1,i)+s(:,i)'*s_dot(:,i)),-2*(s_dot(2,i)*s(2,i)+s(:,i)'*s_dot(:,i)),-2*s_dot(1,i)*s(2,i),-2*s_dot(2,i)*s(1,i),-s_dot(1,i)^2, ...
        -2*s_dot(1,i)*s_dot(2,i),-s_dot(2,i)^2,-s(1,i)^2,-2*s(1,i)*s(2,i),-s(2,i)^2,2*s_dot(:,i)',2*s(:,i)',-1];
    b = [b;(s(:,i)'*s_dot(:,i))^2];
end

W = inv(Q);

for k=1:2
    B = [];
    W = W/trace(W);

    cvx_clear
    cvx_begin sdp
    cvx_solver sdpt3
    cvx_quiet(1)
    cvx_precision best
    variable Y(23,23) symmetric
    variable y(23,1)

    minimize ((trace(A'*W*A*Y)-2*b'*W*A*y+b'*W*b)*1e10)

    subject to

    [Y y;y.' 1]>=0;
    y(6) == Y(5,13)+Y(2,8);
    y(6) == Y(1,7)+Y(5,15);
    y(7) == Y(1,5);
    y(8) == Y(2,5);
    y(9) == Y(1,3);
    y(10) == Y(2,4);
    y(11) == Y(1,4);
    y(12) == Y(2,3);
    y(13) == Y(1,1);
    y(14) == Y(1,2);
    y(15) == Y(2,2);
    y(16) == Y(3,3);
    y(17) == Y(3,4);
    y(18) == Y(4,4);
    y(19) == Y(1,9)+Y(2,11);
    y(20) == Y(2,9)+Y(4,15);
    y(21) == Y(3,9)+Y(2,17);
    y(22) == Y(1,17)+Y(2,18);
    y(23) == Y(3,19)+Y(4,20);
    Y(1,8) == Y(2,7);
    Y(1,8) == Y(5,14);
    Y(1,11) == Y(4,13);
    Y(3,17) == Y(4,16);
    Y(4,8) == Y(5,10);
    Y(10,19) == Y(11,20);
    Y(10,19) == Y(14,22);
    Y(11,19) == Y(13,22);
    Y(1,15) == Y(2,14);
    Y(1,20) == Y(2,19);
    Y(7,14) == Y(8,13);
    Y(7,20) == Y(8,19);
    Y(11,11) == Y(13,18);
    Y(12,12) == Y(15,16);
    Y(12,20) == Y(15,21);
    Y(12,21) == Y(16,20);
    Y(13,15) == Y(14,14);
    Y(3,6) == Y(7,9)+Y(8,12);
    Y(4,6) == Y(7,11)+Y(8,10);
    Y(5,6) == Y(7,7)+Y(8,8);
    Y(5,19) == Y(7,9)+Y(8,11);
    Y(5,20) == Y(7,12)+Y(8,10);
    Y(5,21) == Y(7,16)+Y(8,17);
    Y(5,22) == Y(7,17)+Y(8,18);
    Y(1,9) == Y(3,13);
    Y(1,10) == Y(2,11);
    Y(1,10) == Y(4,14);
    Y(1,12) == Y(2,9);
    Y(1,12) == Y(3,14);
    Y(2,10) == Y(4,15);
    Y(1,16) == Y(3,9);
    Y(2,17) == Y(3,10);
    Y(2,17) == Y(4,12);
    Y(1,17) == Y(3,11);
    Y(1,17) == Y(4,9);
    Y(2,18) == Y(4,10);
    Y(2,22) == Y(4,20);
    Y(1,21) == Y(3,19);
    Y(1,23) == Y(9,19)+Y(11,20);
    Y(9,19) == Y(13,21);
    Y(11,20) == Y(14,22);
    Y(2,19) == Y(9,14)+Y(11,15);
    Y(9,14) == Y(12,13);
    Y(10,14) == Y(11,15);
    Y(2,21) == Y(14,16)+Y(10,12);
    Y(9,12) == Y(14,16);
    Y(10,12) == Y(15,17);
    Y(2,23) == Y(9,20)+Y(15,22);
    Y(12,19) == Y(9,20);
    Y(12,19) == Y(14,21);
    Y(10,20) == Y(15,22);
    Y(3,19) == Y(13,16)+Y(9,10);
    Y(9,9) == Y(13,16);
    Y(3,23) == Y(16,19)+Y(17,20);
    Y(12,22) == Y(17,20);
    Y(12,22) == Y(10,21);
    Y(16,19) == Y(9,21);
    Y(4,19) == Y(13,17)+Y(10,11);
    Y(9,11) == Y(13,17);
    Y(10,11) == Y(14,18);
    Y(4,20) == Y(14,17)+Y(15,18);
    Y(10,10) == Y(15,18);
    Y(9,10) == Y(11,12);
    Y(9,10) == Y(14,17);
    Y(4,21) == Y(9,17)+Y(12,18);
    Y(9,17) == Y(11,16);
    Y(10,17) == Y(12,18);
    Y(4,23) == Y(17,19)+Y(18,20);
    Y(11,21) == Y(17,19);
    Y(18,20) == Y(10,22);
    Y(1,19) == Y(9,13)+Y(11,14);
    Y(10,13) == Y(11,14);
    Y(2,20) == Y(12,14)+Y(10,15);
    Y(9,15) == Y(12,14);

    cvx_end

    p = y(1:2);
    v = y(3:4);
    c = sqrt(y(5));
    for j=1:N
        B = blkdiag(B,2*d(j)*c^2/f0*norm(p-s(:,j))^2);
    end
    W = inv(B'*Q*B);
end
YY=Y-y*y';

rank=0;
lambda=[];
lambda=eig(Y);
if lambda(23)/lambda(22)>=1e6
    rank=1;
end

p1 = y(1:2);
v1 = y(3:4);
c1 = sqrt(y(5));

end

